package com.ym.mec.biz.dal.dao;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.ibatis.annotations.Param;

import com.ym.mec.biz.dal.entity.MusicGroupPaymentCalender;
import com.ym.mec.biz.dal.enums.PaymentStatusEnum;
import com.ym.mec.common.dal.BaseDAO;

public interface MusicGroupPaymentCalenderDao extends BaseDAO<Long, MusicGroupPaymentCalender> {

    /**
     * 根据乐团编号删除乐团缴费周期
     *
     * @param musicGroupId
     */
    void delByGroupId(String musicGroupId);

    /**
     * 根据乐团编号获取乐团缴费日历
     *
     * @param musicGroupId
     * @return
     */
    List<MusicGroupPaymentCalender> findByMusicGroupId(String musicGroupId);

    /**
     * 批量新增乐团缴费周期
     *
     * @param calender
     * @param musicGroupId
     */
    void batchAdd(@Param("calender") List<Integer> calender, @Param("musicGroupId") String musicGroupId);

    /**
     * 批量修改
     *
     * @param musicGroupPaymentCalenderList
     * @return
     */
    int batchUpdate(List<MusicGroupPaymentCalender> musicGroupPaymentCalenderList);

    /**
     * 查询指定状态的记录
     *
     * @param status
     * @return
     */
    List<MusicGroupPaymentCalender> queryByPaymentStatus(@Param("status") List<PaymentStatusEnum> status);

    /**
     * 获取缴费信息锁
     *
     * @param id
     * @return
     */
    MusicGroupPaymentCalender getForLock(@Param("id") Long id);

    /**
     * 根据指定乐团的缴费日期查询交集
     *
     * @param musicGroupId        乐团编号
     * @param startPaymentDate    开始缴费日期
     * @param deadlinePaymentDate 截止缴费日期
     * @return
     */
    int queryIntersectionByPaymentDate(@Param("musicGroupId") String musicGroupId, @Param("startPaymentDate") Date startPaymentDate,
                                       @Param("deadlinePaymentDate") Date deadlinePaymentDate);

    /**
     * 统计预计缴费人数
     *
     * @param collect
     * @return
     */
    List<Map<Long, Long>> countExpectNum(@Param("collect") Set<Long> collect);

    /**
     * 统计预计缴费人数
     *
     * @param collect
     * @return
     */
    List<Map<Long, Long>> countActualNum(@Param("collect") Set<Long> collect);

    /**
     * 乐团最后一次交费信息
     * @param musicGroupId
     * @return
     */
    MusicGroupPaymentCalender getLastStartOne(@Param("musicGroupId") String musicGroupId);

    int queryIntersectionByDate(@Param("musicGroupId") String musicGroupId, @Param("startTime") String startTime, @Param("endTime") String endTime);
}