package com.ym.mec.biz.dal.dao;

import com.ym.mec.biz.dal.dto.SimpleUserDto;
import com.ym.mec.biz.dal.entity.MusicGroupPaymentCalenderDetail;
import com.ym.mec.common.dal.BaseDAO;
import org.apache.ibatis.annotations.Param;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;
import java.util.Set;

public interface MusicGroupPaymentCalenderDetailDao extends BaseDAO<Long, MusicGroupPaymentCalenderDetail> {

	/**
	 * 批量新增
	 * @param musicGroupPaymentCalenderDetailList
	 * @return
	 */
	int batchInsert(List<MusicGroupPaymentCalenderDetail> musicGroupPaymentCalenderDetailList);

	/**
	 * 获取列表
	 * @param ids
	 * @return
	 */
    List<MusicGroupPaymentCalenderDetail> queryListByIds(@Param("ids") String ids);

	/**
	 * 批量修改
	 * @param calenderDetails
	 */
	void batchUpdate(@Param("calenderDetails") List<MusicGroupPaymentCalenderDetail> calenderDetails);

	/**
	 * 根据指定的日历编号查询
	 * @param musicGroupPaymentCalenderId
	 * @return
	 */
	List<MusicGroupPaymentCalenderDetail> queryByCalenderId(Long musicGroupPaymentCalenderId);

	/**
	 * 根据声部获取乐团下完成缴费的学员
	 * @param musicGroupId
	 * @param subjectId
	 * @return
	 */
	List<SimpleUserDto> findMusicGroupStudentWithSubject(@Param("musicGroupId") String musicGroupId,
														 @Param("subjectId") Integer subjectId);

	/**
	 * 统计实际收款金额
	 * @param id
	 * @return
	 */
	BigDecimal sumActualAmount(@Param("id") Long id);

	/**
	 * 获取学生最后一期续费记录
	 * @param musicGroupId
	 * @param userId
	 * @return
	 */
	MusicGroupPaymentCalenderDetail getUserLastCalenderDetail(@Param("musicGroupId") String musicGroupId, @Param("userId") Integer userId);

	/**
	 * 获取可开启缴费的数据集
	 * @param ids
	 * @return
	 */
	List<MusicGroupPaymentCalenderDetail> queryCanOpenList(@Param("ids") String ids);

	/**
	 * 当前用户是否有进行中的订单
	 * @param musicGroupId
	 * @param userId
	 * @return
	 */
	int queryIntersectionByPaymentDate(@Param("musicGroupId") String musicGroupId, @Param("userId") Integer userId);

	/**
	 * 刷新学员乐团付费状态
	 */
	void refreshUserMusicGroupPaymentStatusTask();

	/**
	 * 获取可以推送缴费提醒的列表
	 * @param musicGroupPaymentCalenderId
	 * @return
	 */
	List<MusicGroupPaymentCalenderDetail> queryCanPushList(@Param("musicGroupPaymentCalenderId") Long musicGroupPaymentCalenderId);

	/**
	 * 获取当天创建，已开启缴费，并且未缴费的用户列表
	 * @return
	 */
	List<MusicGroupPaymentCalenderDetail> queryNoPaymentCanPushList();

	/**
	 * 获取学员对应的乐团
	 * @return
	 */
	List<Map<Integer, String>> queryPaymentPushMap();

	/**
	 * 删除缴费详情
	 * @param musicGroupPaymentCalenderIds
	 */
	void batchDel(String musicGroupPaymentCalenderIds);

	/**
	 * 删除未缴费的明细
	 * @param userId
	 * @param musicGroupId
	 */
	void deleteByUserIdAndMusicGroupId(@Param("userId") Integer userId, @Param("musicGroupId") String musicGroupId);

	/**
	 * 根据缴费记录删除缴费详情
	 * @param id
	 */
    void deleteByCalenderId(@Param("id") Long id);

	/**
	 * 获取缴费详情学员列表
	 * @param calenderId
	 * @return
	 */
	Set<Integer> queryStudentIds(Long calenderId);

	/**
	 * 查询进行中的缴费
	 * @param musicGroupId
	 * @param userId
	 * @return
	 */
    Integer countOpenPayment(@Param("musicGroupId") String musicGroupId, @Param("userId") Integer userId);
}