package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.ym.mec.biz.dal.dao.GoodsCategoryDao;
import com.ym.mec.biz.dal.dao.GoodsDao;
import com.ym.mec.biz.dal.entity.Goods;
import com.ym.mec.biz.dal.enums.GoodsType;
import com.ym.mec.biz.service.GoodsService;
import com.ym.mec.biz.service.UploadFileService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.entity.UploadReturnBean;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.excel.IniFileUtil;
import com.ym.mec.util.excel.POIUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.PictureData;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Service
public class GoodsServiceImpl extends BaseServiceImpl<Integer, Goods>  implements GoodsService {
	private static final Logger LOGGER = LoggerFactory.getLogger(CourseScheduleTeacherSalaryServiceImpl.class);
	
	@Autowired
	private GoodsDao goodsDao;
	@Autowired
	private UploadFileService uploadFileService;
	@Autowired
	private GoodsCategoryDao goodsCategoryDao;

	@Override
	public BaseDAO<Integer, Goods> getDAO() {
		return goodsDao;
	}

	@Override
	public List<Goods> findGoodsBySubId(Integer subjectId,String type) {
		return goodsDao.findGoodsBySubId(subjectId,type);
	}

	public int findGoodsNumByCategoryId(Integer goodsCategoryId){
		return goodsDao.findGoodsNumByCategoryId(goodsCategoryId);
	}

	@Override
	public List<Goods> findGoodsByIds(String ids) {
		return goodsDao.findGoodsByIds(ids);
	}

	@Override
	public List<Goods> findTypeGoods(String type) {
		return goodsDao.findTypeGoods(type);
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public List<Goods> importGoods(MultipartFile file) throws Exception {
		Map<String, List<Map<String, Object>>> sheetsListMap = POIUtil.importExcel(file.getInputStream(), 2, file.getOriginalFilename());
		InputStream inputStream = new ClassPathResource("columnMapper.ini").getInputStream();
		Map<String,String> columns = IniFileUtil.readIniFile(inputStream);
		List<Goods> goodsList = new ArrayList<>();
		Map<String, Integer> map = MapUtil.convertIntegerMap(goodsCategoryDao.queryCategotyMap());
		for (String e : sheetsListMap.keySet()) {
			List<Map<String, Object>> sheet = sheetsListMap.get(e);
			valueIsNull: for (Map<String, Object> row : sheet) {
				if (row.size() == 0){
					continue;
				}
				JSONObject objectMap = new JSONObject();
				for (String s : row.keySet()) {
					if(!columns.containsKey(s)){
						continue;
					}
					String columnValue = columns.get(s);
					if(null == row.get(s) || StringUtils.isEmpty(row.get(s).toString())){
						LOGGER.error("商品导入异常：参数{}不可为空 param:{}",columnValue,objectMap);
						continue valueIsNull;
					}
					if (columnValue.equals("type")) {
						for (GoodsType goodsType : GoodsType.values()) {
							if (goodsType.getDesc().equals(row.get(s).toString())) {
								objectMap.put(columnValue, goodsType);
								break;
							}
						}
						continue;
					}
					if (columnValue.equals("goodsCategoryName")) {
						Integer integer = map.get(row.get(s));
						if(integer == null){
							LOGGER.error("商品导入异常：商品分类解析失败 param:{}",objectMap);
							continue valueIsNull;
						}
						objectMap.put("goodsCategoryId", integer);
						continue;
					}
					if (columnValue.equals("image")) {
						List<PictureData> pictureDataList = (ArrayList<PictureData>) row.get(s);
						if(pictureDataList == null || pictureDataList.size() == 0){
							continue valueIsNull;
						}
						PictureData pictureData = pictureDataList.get(0);
						InputStream in = new ByteArrayInputStream(pictureData.getData());
						UploadReturnBean bean = uploadFileService.uploadFile(in, pictureData.suggestFileExtension());
						if (!bean.isStatus()) {
							LOGGER.error("商品导入异常：上传图片失败 param:{}",objectMap);
							continue valueIsNull;
						}
						objectMap.put(columnValue, bean.getUrl());
						continue;
					}
					objectMap.put(columnValue, row.get(s));
				}
				Goods goods = null;
				try {
					goods = JSONObject.parseObject(objectMap.toJSONString(),Goods.class);
					goodsList.add(goods);
				} catch (Exception ex) {
					throw new BizException("导入数据出错");
				}
			}
		}
		if(goodsList.size() != 0){
			goodsDao.batchInsert(goodsList);
		}
		return goodsList;
	}
}