package com.ym.mec.biz.service.impl;

import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.FeeStudentDto;
import com.ym.mec.biz.dal.dto.SimpleUserDto;
import com.ym.mec.biz.dal.entity.MusicGroup;
import com.ym.mec.biz.dal.entity.MusicGroupPaymentCalender;
import com.ym.mec.biz.dal.entity.MusicGroupPaymentCalenderDetail;
import com.ym.mec.biz.dal.entity.MusicGroupStudentFee;
import com.ym.mec.biz.dal.enums.MessageTypeEnum;
import com.ym.mec.biz.dal.enums.YesOrNoEnum;
import com.ym.mec.biz.service.MusicGroupPaymentCalenderDetailService;
import com.ym.mec.biz.service.SysConfigService;
import com.ym.mec.biz.service.SysMessageService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

import static com.ym.mec.biz.dal.entity.MusicGroupStudentFee.PaymentStatus.NON_PAYMENT;
import static com.ym.mec.biz.dal.entity.MusicGroupStudentFee.PaymentStatus.PAID_COMPLETED;
import static com.ym.mec.biz.dal.enums.PaymentStatusEnum.OPEN;

@Service
public class MusicGroupPaymentCalenderDetailServiceImpl extends BaseServiceImpl<Long, MusicGroupPaymentCalenderDetail>  implements MusicGroupPaymentCalenderDetailService {
	
	@Autowired
	private MusicGroupPaymentCalenderDetailDao musicGroupPaymentCalenderDetailDao;
	@Autowired
	private MusicGroupPaymentCalenderDao musicGroupPaymentCalenderDao;
	@Autowired
	private MusicGroupDao musicGroupDao;
	@Autowired
	private SysConfigDao sysConfigDao;
	@Autowired
	private MusicGroupStudentFeeDao musicGroupStudentFeeDao;
	@Autowired
	private SysMessageService sysMessageService;

	@Override
	public BaseDAO<Long, MusicGroupPaymentCalenderDetail> getDAO() {
		return musicGroupPaymentCalenderDetailDao;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void updateExpectAmount(BigDecimal expectAmount,String ids) {
		if(expectAmount == null || StringUtils.isEmpty(ids)){
			throw new BizException("参数校验失败");
		}
		//获取缴费列表
		List<MusicGroupPaymentCalenderDetail> calenderDetails = musicGroupPaymentCalenderDetailDao.queryListByIds(ids);
		if(calenderDetails.size() == 0){
			throw new BizException("缴费记录不存在");
		}
		Date date = new Date();
		MusicGroupPaymentCalender calender = musicGroupPaymentCalenderDao.get(calenderDetails.get(0).getMusicGroupPaymentCalenderId());
		Set<Integer> studentIds = new HashSet<>();
				calenderDetails.forEach(e->{
			if(e.getPaymentStatus() == null || e.getPaymentStatus() != NON_PAYMENT){
				throw new BizException("修改失败,缴费状态不匹配");
			}
			if(expectAmount.doubleValue() != 0){

			}
			/*if(expectAmount.doubleValue() == 0){
				e.setPaymentStatus(PAID_COMPLETED);
			}*/
			e.setUpdateTime(date);
			e.setExpectAmount(expectAmount);
			if(expectAmount.doubleValue() > 0){
				if((e.getOpen() != null && e.getOpen() == 1) || calender.getPaymentStatus() == OPEN){
					studentIds.add(e.getUserId());
				}
			}
		});
		/*if(expectAmount.doubleValue() == 0){
			//如果是设置为0元
			MusicGroupPaymentCalender calender = musicGroupPaymentCalenderDao.get(calenderDetails.get(0).getMusicGroupPaymentCalenderId());
			if(calender.getType() == ONLINE){
				//剩余网管课可购买次数加一
				musicGroupStudentFeeDao.remainNetworkClassTimesAdd(ids);
			}
			//缴费完成
			musicGroupStudentFeeDao.setPaidCompleted(calenderDetails);
		}*/
		musicGroupPaymentCalenderDetailDao.batchUpdate(calenderDetails);
//		if(expectAmount.doubleValue() > 0){
//			Set<Integer> studentIds = calenderDetails.stream().map(e -> e.getUserId()).collect(Collectors.toSet());
			//推送消息
			if (studentIds.size() > 0) {
				String musicGroupId = calenderDetails.get(0).getMusicGroupId();
				Map<Integer, String> push = new HashMap<>();
				for (Integer userId : studentIds) {
					push.put(userId, userId + "");
				}
				MusicGroup musicGroup = musicGroupDao.get(musicGroupId);
				String baseUrl = sysConfigDao.findConfigValue(SysConfigService.BASE_API_URL);
				String memo = "4?" + baseUrl + "/#/renew?musicGroupId=" + musicGroupId;
				// 发送续费通知
				sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.STUDENT_PUSH_WAIT_RENEW_MESSAGE, push, null, 0, memo,"STUDENT",musicGroup.getName());
			}
//		}
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public String openPayment(String ids) {
		if(StringUtils.isEmpty(ids)){
			throw new BizException("参数校验失败");
		}
		//获取可开启缴费列表
		List<MusicGroupPaymentCalenderDetail> calenderDetails = musicGroupPaymentCalenderDetailDao.queryCanOpenList(ids);
		if(calenderDetails.size() == 0){
			return "操作失败，没有可以开启缴费的学员";
		}
//		Long musicGroupPaymentCalenderId = calenderDetails.get(0).getMusicGroupPaymentCalenderId();
//		MusicGroupPaymentCalender calender = musicGroupPaymentCalenderDao.get(musicGroupPaymentCalenderId);
//		Date startPaymentDate = calender.getStartPaymentDate();
//		Date deadlinePaymentDate = calender.getDeadlinePaymentDate();
		int length = ids.split(",").length;
		Date date = new Date();
		Iterator<MusicGroupPaymentCalenderDetail> iterator = calenderDetails.iterator();
		MusicGroupPaymentCalenderDetail next;
		while(iterator.hasNext()){
			next = iterator.next();
			Integer countOpenPayment = musicGroupPaymentCalenderDetailDao.countOpenPayment(next.getMusicGroupId(), next.getUserId());
			if(countOpenPayment > 0){
				iterator.remove();
			}
			/*int count1 = musicGroupPaymentCalenderDao.queryIntersectionByPaymentDate(next.getMusicGroupId(),startPaymentDate,deadlinePaymentDate);
			int count = musicGroupPaymentCalenderDetailDao.queryIntersectionByPaymentDate(next.getMusicGroupId(),next.getUserId());
			if(count > 0 || count1 > 0){
				iterator.remove();
			}*/
		}
		if(calenderDetails.size() == 0){
			return "操作失败，学员有未完成的订单";
		}
		String musicGroupId = calenderDetails.get(0).getMusicGroupId();
		calenderDetails.forEach(e->{
			e.setUpdateTime(date);
			e.setOpen(1);
//			e.setPaymentStatus(PROCESSING);
			e.setStartPaymentDate(date);
			e.setDeadlinePaymentDate(date);
		});
		musicGroupPaymentCalenderDetailDao.batchUpdate(calenderDetails);
		//修改学员缴费状态为未缴费
		musicGroupStudentFeeDao.setNoPayment(calenderDetails);
		Set<Integer> studentIds = calenderDetails.stream().map(e -> e.getUserId()).collect(Collectors.toSet());
		//推送消息
		if (studentIds.size() > 0) {
			Map<Integer, String> push = new HashMap<>();
			for (Integer userId : studentIds) {
				push.put(userId, userId + "");
			}
			MusicGroup musicGroup = musicGroupDao.get(musicGroupId);
			String baseUrl = sysConfigDao.findConfigValue(SysConfigService.BASE_API_URL);
			String memo = "4?" + baseUrl + "/#/renew?musicGroupId=" + musicGroupId;
			// 发送续费通知
			sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.STUDENT_PUSH_WAIT_RENEW_MESSAGE, push, null, 0, memo,"STUDENT",musicGroup.getName());
		}
		StringBuffer append = new StringBuffer("操作成功: 成功开启缴费").append(studentIds.size()).append("条,失败").append(length - studentIds.size()).append("条");
		return append.toString();
	}

	@Override
	public List<SimpleUserDto> findMusicGroupStudentWithSubject(String musicGroupId, Integer subjectId) {
		if(Objects.isNull(musicGroupId)){
			throw new BizException("请指定乐团");
		}
		return musicGroupPaymentCalenderDetailDao.findMusicGroupStudentWithSubject(musicGroupId, subjectId);
	}

	@Override
	public void refreshUserMusicGroupPaymentStatusTask() {
		musicGroupPaymentCalenderDetailDao.refreshUserMusicGroupPaymentStatusTask();
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void add(Long musicGroupPaymentCalenderId, Integer userId) {
		if(musicGroupPaymentCalenderId == null || userId == null){
			throw new BizException("参数校验异常");
		}
		MusicGroupPaymentCalender calender = musicGroupPaymentCalenderDao.get(musicGroupPaymentCalenderId);
		if(calender == null){
			throw new BizException("缴费信息不存在");
		}
		MusicGroupPaymentCalenderDetail userLastCalenderDetail = musicGroupPaymentCalenderDetailDao.getUserLastCalenderDetail(calender.getMusicGroupId(), userId);
		if(userLastCalenderDetail != null){
			throw new BizException("缴费信息已存在");
		}
		Date date = new Date();
		Integer countOpenPayment = musicGroupPaymentCalenderDetailDao.countOpenPayment(calender.getMusicGroupId(), userId);
		MusicGroupStudentFee studentFee = musicGroupStudentFeeDao.findByUser(userId, calender.getMusicGroupId());
		if(countOpenPayment > 0){
			throw new BizException("操作失败: 学员有未完成的缴费");
		}
		calender.setUpdateTime(date);
		//生成详情
		MusicGroupPaymentCalenderDetail musicGroupPaymentCalenderDetail = new MusicGroupPaymentCalenderDetail();
		musicGroupPaymentCalenderDetail.setMusicGroupPaymentCalenderId(calender.getId());
		musicGroupPaymentCalenderDetail.setCreateTime(date);
		musicGroupPaymentCalenderDetail.setExpectAmount(studentFee.getCourseFee());
		studentFee.setUpdateTime(date);
		/*if (studentFee.getCourseFee().doubleValue() == 0) {
			studentFee.setPaymentStatus(PAID_COMPLETED);
			musicGroupPaymentCalenderDetail.setPaymentStatus(PAID_COMPLETED);
			if(calender.getType() == ONLINE){
				studentFee.setRemainNetworkClassTimes(studentFee.getRemainNetworkClassTimes() + 1);
//				musicGroupStudentFeeDao.remainNetworkClassTimesAdd(userLastCalenderDetail.getId().toString());
			}
		} else {
			studentFee.setPaymentStatus(NON_PAYMENT);
			musicGroupPaymentCalenderDetail.setPaymentStatus(MusicGroupStudentFee.PaymentStatus.NON_PAYMENT);
		}*/
		studentFee.setPaymentStatus(NON_PAYMENT);
		musicGroupPaymentCalenderDetail.setPaymentStatus(MusicGroupStudentFee.PaymentStatus.NON_PAYMENT);
		musicGroupPaymentCalenderDetail.setUpdateTime(date);
		musicGroupPaymentCalenderDetail.setUserId(userId);
		musicGroupPaymentCalenderDetailDao.insert(musicGroupPaymentCalenderDetail);
		musicGroupPaymentCalenderDao.update(calender);
		musicGroupStudentFeeDao.update(studentFee);
	}

	@Override
	public List<FeeStudentDto> queryFeeStudents(String musicGroupId,String search,Integer subjectId) {
		return musicGroupStudentFeeDao.queryFeeStudents(musicGroupId,search,subjectId);
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void batchDel(String musicGroupPaymentCalenderIds) {
		List<MusicGroupPaymentCalenderDetail> details = musicGroupPaymentCalenderDetailDao.queryListByIds(musicGroupPaymentCalenderIds);
		if(details == null || details.size() == 0){
			throw new BizException("删除失败: 数据不可为空");
		}
		long count = details.stream().filter(e -> e.getPaymentStatus() == PAID_COMPLETED).count();
		if(count > 0){
			throw new BizException("删除失败: 存在已缴费的学员");
		}
		musicGroupPaymentCalenderDetailDao.batchDel(musicGroupPaymentCalenderIds);
	}
}