package com.ym.mec.biz.service.impl;

import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.entity.MusicGroup;
import com.ym.mec.biz.dal.entity.MusicGroupPaymentCalender;
import com.ym.mec.biz.dal.entity.MusicGroupPaymentCalenderDetail;
import com.ym.mec.biz.dal.entity.MusicGroupStudentFee;
import com.ym.mec.biz.dal.entity.MusicGroupStudentFee.PaymentStatus;
import com.ym.mec.biz.dal.enums.MessageTypeEnum;
import com.ym.mec.biz.dal.enums.PaymentStatusEnum;
import com.ym.mec.biz.dal.page.MusicCalenderQueryInfo;
import com.ym.mec.biz.service.MusicGroupPaymentCalenderService;
import com.ym.mec.biz.service.SysConfigService;
import com.ym.mec.biz.service.SysMessageService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext;
import com.ym.mec.util.collection.MapUtil;
import com.ym.mec.util.date.DateUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

import static com.ym.mec.biz.dal.enums.PaymentStatusEnum.NO;

@Service
public class MusicGroupPaymentCalenderServiceImpl extends BaseServiceImpl<Long, MusicGroupPaymentCalender> implements MusicGroupPaymentCalenderService {

	@Autowired
	private MusicGroupPaymentCalenderDao musicGroupPaymentCalenderDao;

	@Autowired
	private MusicGroupPaymentCalenderDetailDao musicGroupPaymentCalenderDetailDao;

	@Autowired
	private MusicGroupStudentFeeDao musicGroupStudentFeeDao;
	@Autowired
	private MusicGroupDao musicGroupDao;
	@Autowired
	private SysConfigDao sysConfigDao;
	@Autowired
	private SysMessageService sysMessageService;

	@Override
	public BaseDAO<Long, MusicGroupPaymentCalender> getDAO() {
		return musicGroupPaymentCalenderDao;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean create(MusicGroupPaymentCalender musicGroupPaymentCalender) {
		String musicGroupId = musicGroupPaymentCalender.getMusicGroupId();

		musicGroupDao.getLocked(musicGroupId);

		Date date = new Date();
		List<MusicGroupStudentFee> musicGroupStudentFeeList = null;

		musicGroupPaymentCalender.setCreateTime(date);
		musicGroupPaymentCalender.setUpdateTime(date);
		if (musicGroupPaymentCalender.getDeadlinePaymentDate() == null) {
			musicGroupPaymentCalender.setDeadlinePaymentDate(DateUtil.addDays(musicGroupPaymentCalender.getStartPaymentDate(), 3));
		}
		// 判断缴费开始时间、结束时间是否被其他缴费记录占用
//		int count = musicGroupPaymentCalenderDao.queryIntersectionByPaymentDate(musicGroupId, musicGroupPaymentCalender.getStartPaymentDate(),
//				musicGroupPaymentCalender.getDeadlinePaymentDate());

		String format = DateUtil.format(musicGroupPaymentCalender.getStartPaymentDate(), DateUtil.DEFAULT_PATTERN);
		String format1 = DateUtil.format(musicGroupPaymentCalender.getDeadlinePaymentDate(), DateUtil.DEFAULT_PATTERN);
		int count = musicGroupPaymentCalenderDao.queryIntersectionByDate(musicGroupId, format,format1);
		if (count > 0) {
			throw new BizException("缴费时间存在冲突，请修改缴费开始日期");
		}
		if (date.after(musicGroupPaymentCalender.getDeadlinePaymentDate())) {
			musicGroupPaymentCalender.setPaymentStatus(PaymentStatusEnum.YES);
		} else if (date.after(musicGroupPaymentCalender.getStartPaymentDate())) {
			musicGroupPaymentCalender.setPaymentStatus(PaymentStatusEnum.OPEN);
		} else {
			musicGroupPaymentCalender.setPaymentStatus(PaymentStatusEnum.NO);
		}
		// 统计缴费人数
		musicGroupStudentFeeList = musicGroupStudentFeeDao.queryByMusicGroupId(musicGroupId,musicGroupPaymentCalender.getStudentIds());
		if (musicGroupStudentFeeList == null) {
			musicGroupStudentFeeList = new ArrayList<>();
		}
		musicGroupPaymentCalender.setExpectNum(musicGroupStudentFeeList.size());

		this.insert(musicGroupPaymentCalender);

		List<MusicGroupStudentFee> updateMusicGroupStudentFeeList = new ArrayList<MusicGroupStudentFee>();

		List<MusicGroupPaymentCalenderDetail> musicGroupPaymentCalenderDetailList = new ArrayList<MusicGroupPaymentCalenderDetail>();

		if (musicGroupStudentFeeList != null && musicGroupStudentFeeList.size() > 0) {
			Set<Integer> studentIds = new HashSet<>();
			MusicGroupPaymentCalenderDetail musicGroupPaymentCalenderDetail = null;
			// 创建缴费明细
			for (MusicGroupStudentFee mgsf : musicGroupStudentFeeList) {
				musicGroupPaymentCalenderDetail = new MusicGroupPaymentCalenderDetail();
				musicGroupPaymentCalenderDetail.setMusicGroupPaymentCalenderId(musicGroupPaymentCalender.getId());
				musicGroupPaymentCalenderDetail.setCreateTime(date);
				musicGroupPaymentCalenderDetail.setExpectAmount(mgsf.getCourseFee());
				musicGroupPaymentCalenderDetail.setPaymentStatus(PaymentStatus.NON_PAYMENT);
				if (mgsf.getCourseFee().doubleValue() != 0) {
					studentIds.add(mgsf.getUserId());
				}
				musicGroupPaymentCalenderDetail.setUpdateTime(date);
				musicGroupPaymentCalenderDetail.setUserId(mgsf.getUserId());
				musicGroupPaymentCalenderDetailList.add(musicGroupPaymentCalenderDetail);
			}
			if (musicGroupPaymentCalender.getPaymentStatus() == PaymentStatusEnum.OPEN) {
				// 更新学员缴费状态
				for (MusicGroupStudentFee mgsf : musicGroupStudentFeeList) {
					mgsf.setPaymentStatus(PaymentStatus.NON_PAYMENT);
					mgsf.setUpdateTime(date);
					updateMusicGroupStudentFeeList.add(mgsf);
				}
				if (studentIds.size() > 0) {
					Map<Integer, String> push = new HashMap<>();
					for (Integer userId : studentIds) {
						push.put(userId, userId + "");
					}
					MusicGroup musicGroup = musicGroupDao.get(musicGroupPaymentCalender.getMusicGroupId());
					String configValue = sysConfigDao.findConfigValue(SysConfigService.BASE_API_URL);
					String memo = "4?" + configValue + "/#/renew?musicGroupId=" + musicGroup.getId();
					// 发送续费通知
					sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.STUDENT_PUSH_WAIT_RENEW_MESSAGE, push,
							null, 0, memo, "STUDENT", musicGroup.getName());
				}
			}
		}

		if (musicGroupPaymentCalenderDetailList.size() > 0) {
			musicGroupPaymentCalenderDetailDao.batchInsert(musicGroupPaymentCalenderDetailList);
		}

		if (updateMusicGroupStudentFeeList.size() > 0) {
			musicGroupStudentFeeDao.batchUpdate(updateMusicGroupStudentFeeList);
		}

		return true;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public boolean autoUpdateMusicGroupPaymentCalenderStatus() {
		// 获取当天创建且未缴费的数据，并推送
		List<MusicGroupPaymentCalenderDetail> details = musicGroupPaymentCalenderDetailDao.queryNoPaymentCanPushList();
		if (details != null && details.size() > 0) {
			Set<String> musicGroupIds = details.stream().map(e -> e.getMusicGroupId()).collect(Collectors.toSet());
			Map<String, String> musicGroupNameMap = MapUtil.convertMybatisMap(musicGroupDao.queryMusicGroupNameMap(musicGroupIds));
			for (MusicGroupPaymentCalenderDetail detail : details) {
				if (detail.getExpectAmount().doubleValue() == 0) {
					continue;
				}
				Map<Integer, String> push = new HashMap<>(1);
				push.put(detail.getUserId(), detail.getUserId() + "");
				// MusicGroup musicGroup = musicGroupDao.get(detail.getMusicGroupId());
				String baseUrl = sysConfigDao.findConfigValue(SysConfigService.BASE_API_URL);
				String memo = "4?" + baseUrl + "/#/renew?musicGroupId=" + detail.getMusicGroupId();
				// 发送续费通知
				sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.STUDENT_PUSH_WAIT_RENEW_MESSAGE, push,
						null, 0, memo, "STUDENT", musicGroupNameMap.get(detail.getMusicGroupId()));
			}
		}

		Date date = new Date();
		List<PaymentStatusEnum> statusList = new ArrayList<PaymentStatusEnum>();
		statusList.add(PaymentStatusEnum.NO);
		statusList.add(PaymentStatusEnum.OPEN);
		List<MusicGroupPaymentCalender> musicGroupPaymentCalenderList = musicGroupPaymentCalenderDao.queryByPaymentStatus(statusList);

		List<MusicGroupPaymentCalender> updateMusicGroupPaymentCalenderList = new ArrayList<MusicGroupPaymentCalender>();

		MusicGroupPaymentCalenderDetail musicGroupPaymentCalenderDetail = null;
		MusicGroupStudentFee musicGroupStudentFee = null;
		List<MusicGroupStudentFee> updateMusicGroupStudentFeeList = new ArrayList<MusicGroupStudentFee>();
		List<MusicGroupPaymentCalenderDetail> musicGroupPaymentCalenderDetailList = new ArrayList<MusicGroupPaymentCalenderDetail>();
		List<MusicGroupPaymentCalenderDetail> updateMusicGroupPaymentCalenderDetailList = new ArrayList<MusicGroupPaymentCalenderDetail>();

		for (MusicGroupPaymentCalender mgpc : musicGroupPaymentCalenderList) {
			Set<Integer> studentIds = musicGroupPaymentCalenderDetailDao.queryStudentIds(mgpc.getId());
			List<MusicGroupStudentFee> musicGroupStudentFeeList = musicGroupStudentFeeDao.queryByMusicGroupId(mgpc.getMusicGroupId(),StringUtils.join(studentIds,","));
			Map<Integer, MusicGroupStudentFee> feeMap = musicGroupStudentFeeList.stream()
					.collect(Collectors.toMap(MusicGroupStudentFee::getUserId, fee -> fee));

			if (mgpc.getDeadlinePaymentDate() != null && date.after(mgpc.getDeadlinePaymentDate()) && mgpc.getPaymentStatus() == PaymentStatusEnum.OPEN) {
				// “进行中”更新至“已结束”
				mgpc.setUpdateTime(date);
				mgpc.setPaymentStatus(PaymentStatusEnum.YES);
				updateMusicGroupPaymentCalenderList.add(mgpc);

				// 0元学生改成“已缴费”
				List<MusicGroupPaymentCalenderDetail> list = musicGroupPaymentCalenderDetailDao.queryByCalenderId(mgpc.getId());
				for (MusicGroupPaymentCalenderDetail mgpcd : list) {
					if (mgpcd.getExpectAmount().doubleValue() == 0) {
						mgpcd.setActualAmount(BigDecimal.ZERO);
						mgpcd.setPaymentStatus(PaymentStatus.PAID_COMPLETED);
						mgpcd.setUpdateTime(date);
						updateMusicGroupPaymentCalenderDetailList.add(mgpcd);

						// 更新fee表，且加1
						musicGroupStudentFee = feeMap.get(mgpcd.getUserId());
						if (musicGroupStudentFee != null) {
							musicGroupStudentFee.setPaymentStatus(PaymentStatus.PAID_COMPLETED);
							musicGroupStudentFee.setUpdateTime(date);
							musicGroupStudentFee.setRemainNetworkClassTimes(musicGroupStudentFee.getContinuousAbsenteeismTimes() + 1);
							updateMusicGroupStudentFeeList.add(musicGroupStudentFee);
						}
					}
				}

			} else if (mgpc.getStartPaymentDate() != null && date.after(mgpc.getStartPaymentDate()) && mgpc.getPaymentStatus() == PaymentStatusEnum.NO) {
				// “未开始”更新至“进行中”
				mgpc.setUpdateTime(date);
				mgpc.setPaymentStatus(PaymentStatusEnum.OPEN);
				updateMusicGroupPaymentCalenderList.add(mgpc);

				// 学生状态更新进行中
//				Set<Integer> studentIds = new HashSet<Integer>();
				// 创建缴费明细
				/*for (MusicGroupStudentFee mgsf : musicGroupStudentFeeList) {
					if (mgsf.getPaymentStatus() != PaymentStatus.PAID_COMPLETED) {
						continue;
					}
					musicGroupPaymentCalenderDetail = new MusicGroupPaymentCalenderDetail();
					musicGroupPaymentCalenderDetail.setMusicGroupPaymentCalenderId(mgpc.getId());
					musicGroupPaymentCalenderDetail.setCreateTime(date);
					musicGroupPaymentCalenderDetail.setExpectAmount(mgsf.getCourseFee());
					musicGroupPaymentCalenderDetail.setPaymentStatus(PaymentStatus.NON_PAYMENT);
					if (mgsf.getCourseFee().doubleValue() != 0) {
						studentIds.add(mgsf.getUserId());
					}
					musicGroupPaymentCalenderDetail.setUpdateTime(date);
					musicGroupPaymentCalenderDetail.setUserId(mgsf.getUserId());

					musicGroupPaymentCalenderDetailList.add(musicGroupPaymentCalenderDetail);
				}*/

				// 更新学员缴费状态
				for (MusicGroupStudentFee mgsf : musicGroupStudentFeeList) {
					mgsf.setPaymentStatus(PaymentStatus.NON_PAYMENT);
					mgsf.setUpdateTime(date);
					updateMusicGroupStudentFeeList.add(mgsf);
				}
				if (studentIds.size() > 0) {
					Map<Integer, String> push = new HashMap<>();
					for (Integer userId : studentIds) {
						push.put(userId, userId + "");
					}
					MusicGroup musicGroup = musicGroupDao.get(mgpc.getMusicGroupId());
					String baseUrl = sysConfigDao.findConfigValue(SysConfigService.BASE_API_URL);
					String memo = "4?" + baseUrl + "/#/renew?musicGroupId=" + musicGroup.getId();
					// 发送续费通知
					sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.STUDENT_PUSH_WAIT_RENEW_MESSAGE, push,
							null, 0, memo, "STUDENT", musicGroup.getName());
				}
			}
		}

		if (updateMusicGroupPaymentCalenderList.size() > 0) {
			musicGroupPaymentCalenderDao.batchUpdate(updateMusicGroupPaymentCalenderList);
		}

		if (musicGroupPaymentCalenderDetailList.size() > 0) {
			musicGroupPaymentCalenderDetailDao.batchInsert(musicGroupPaymentCalenderDetailList);
		}

		if (updateMusicGroupStudentFeeList.size() > 0) {
			musicGroupStudentFeeDao.batchUpdate(updateMusicGroupStudentFeeList);
		}

		if (updateMusicGroupPaymentCalenderDetailList.size() > 0) {
			musicGroupPaymentCalenderDetailDao.batchUpdate(updateMusicGroupPaymentCalenderDetailList);
		}

		return true;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void batchInsert(List<MusicGroupPaymentCalender> musicGroupPaymentCalenders) {
		if (musicGroupPaymentCalenders != null && musicGroupPaymentCalenders.size() > 0) {
			musicGroupPaymentCalenderDao.delByGroupId(musicGroupPaymentCalenders.get(0).getMusicGroupId());
			musicGroupPaymentCalenders.forEach(e -> {
				musicGroupPaymentCalenderDao.insert(e);
			});
		}
	}

	@Override
	public Date getNextPaymentDate(String musicGroupId, Date latestPaidDate, MusicGroupStudentFee fee) {
		List<Integer> months = null;
		if (fee != null) {
			String paymentPeriodList = fee.getPaymentPeriodList();
			if (StringUtils.isEmpty(paymentPeriodList)) {
				return null;
			} else {
				months = Arrays.asList(paymentPeriodList.split(",")).stream().map(x -> Integer.parseInt(x)).collect(Collectors.toList());
				Collections.sort(months);
			}
		} else {
			List<MusicGroupPaymentCalender> musicGroupPaymentCalenderList = musicGroupPaymentCalenderDao.findByMusicGroupId(musicGroupId);
			if (musicGroupPaymentCalenderList != null && musicGroupPaymentCalenderList.size() > 0) {
				months = musicGroupPaymentCalenderList.stream().map(e -> e.getPaymentMonth()).sorted().collect(Collectors.toList());
			}
		}
		if (months != null && months.size() > 0) {
			Date date = latestPaidDate;
			if (date == null) {
				date = new Date();
			}
			// 获取当前月份
			int currentMonth = Integer.parseInt(DateUtil.getMonth(date));
			int nextMonth = months.get(0);
			for (int i = 0; i < months.size(); i++) {
				if (currentMonth < months.get(i)) {
					nextMonth = months.get(i);
					break;
				}
			}
			// 修改学员付费周期
			Date nextPaymentDate = null;
			if (nextMonth > currentMonth) {
				nextPaymentDate = DateUtil.addMonths(date, nextMonth - currentMonth);
			} else if (nextMonth < currentMonth) {
				nextPaymentDate = DateUtil.addMonths(date, 12 - currentMonth + nextMonth);
			} else {
				nextPaymentDate = DateUtil.addMonths(date, 12);
			}
			return DateUtil.getFirstDayOfMonth(nextPaymentDate);
		}
		return null;
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void updateStartTime(Date startTime,Date endTime,Long id) {
		if (startTime == null || id == null || endTime == null) {
			throw new BizException("参数校验失败");
		}
		MusicGroupPaymentCalender calender = musicGroupPaymentCalenderDao.get(id);
		if (calender == null) {
			throw new BizException("缴费信息不存在");
		}
		if (calender.getPaymentStatus() == null || calender.getPaymentStatus() != NO) {
			throw new BizException("修改失败,缴费状态不匹配");
		}
		Date date = new Date();
		// 判断缴费开始时间、结束时间是否被其他缴费记录占用
//		int count = musicGroupPaymentCalenderDao.queryIntersectionByPaymentDate(calender.getMusicGroupId(), startTime,endTime);
		String format = DateUtil.format(startTime, DateUtil.DEFAULT_PATTERN);
		String format1 = DateUtil.format(endTime, DateUtil.DEFAULT_PATTERN);
		int count = musicGroupPaymentCalenderDao.queryIntersectionByDate(calender.getMusicGroupId(), format,format1);
		if (count > 0) {
			throw new BizException("缴费时间存在冲突，请修改缴费开始日期");
		}
		calender.setStartPaymentDate(startTime);
		calender.setDeadlinePaymentDate(endTime);
		calender.setUpdateTime(date);
		if (endTime.before(date) && !DateUtil.isSameDay(date,endTime)) {
			calender.setPaymentStatus(PaymentStatusEnum.YES);
		} else if (date.after(startTime) || DateUtil.isSameDay(date,endTime)) {
			calender.setPaymentStatus(PaymentStatusEnum.OPEN);
			//修改Fee缴费状态为未缴费
			musicGroupStudentFeeDao.updatePaymentStatus(calender.getId(),"NON_PAYMENT");
			List<MusicGroupPaymentCalenderDetail> details = musicGroupPaymentCalenderDetailDao.queryByCalenderId(calender.getId());
			Set<Integer> studentIds = details.stream().map(e -> e.getUserId()).collect(Collectors.toSet());
			if (studentIds.size() > 0) {
				Map<Integer, String> push = new HashMap<>();
				for (Integer userId : studentIds) {
					push.put(userId, userId + "");
				}
				MusicGroup musicGroup = musicGroupDao.get(calender.getMusicGroupId());
				String configValue = sysConfigDao.findConfigValue(SysConfigService.BASE_API_URL);
				String memo = "4?" + configValue + "/#/renew?musicGroupId=" + musicGroup.getId();
				// 发送续费通知
				sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.STUDENT_PUSH_WAIT_RENEW_MESSAGE, push,
						null, 0, memo, "STUDENT", musicGroup.getName());
			}
		} else {
			calender.setPaymentStatus(PaymentStatusEnum.NO);
		}
		musicGroupPaymentCalenderDao.update(calender);
	}

	@Override
	@Transactional(rollbackFor = Exception.class)
	public void del(Long id) {
		if (id == null) {
			throw new BizException("参数校验失败");
		}
		MusicGroupPaymentCalender calender = musicGroupPaymentCalenderDao.get(id);
		if (calender == null) {
			throw new BizException("缴费信息不存在");
		}
		if (calender.getPaymentStatus() == null || calender.getPaymentStatus() != NO) {
			throw new BizException("删除失败,缴费状态不匹配");
		}
		musicGroupPaymentCalenderDao.delete(id);
		musicGroupPaymentCalenderDetailDao.deleteByCalenderId(id);
	}

	@Override
	public Object getDetail(Long id) {
		MusicGroupPaymentCalender calender = musicGroupPaymentCalenderDao.get(id);
		Set<Long> set = new HashSet<>();
		set.add(id);
		Map<Long, Long> expectNumMap = MapUtil.convertIntegerMap(musicGroupPaymentCalenderDao.countExpectNum(set));
		Map<Long, Long> actualNumMap = MapUtil.convertIntegerMap(musicGroupPaymentCalenderDao.countActualNum(set));
		calender.setActualNum(actualNumMap.get(id) == null ? 0 : actualNumMap.get(id).intValue());
		calender.setExpectNum(expectNumMap.get(id) == null ? 0 : expectNumMap.get(id).intValue());
		BigDecimal sumActualAmount = musicGroupPaymentCalenderDetailDao.sumActualAmount(id);
		Map<String, Object> result = new HashMap<>(2);
		result.put("calender", calender);
		result.put("sumActualAmount", sumActualAmount);
		return result;
	}

	@Override
	public PageInfo<MusicGroupPaymentCalender> queryDetailPage(MusicCalenderQueryInfo queryInfo) {
		PageInfo<MusicGroupPaymentCalender> pageInfo = queryPage(queryInfo);
		List<MusicGroupPaymentCalender> rows = pageInfo.getRows();
		if (rows.size() == 0) {
			return pageInfo;
		}
		Set<Long> collect = rows.stream().map(e -> e.getId()).collect(Collectors.toSet());
		Map<Long, Long> expectNumMap = MapUtil.convertIntegerMap(musicGroupPaymentCalenderDao.countExpectNum(collect));
		Map<Long, Long> actualNumMap = MapUtil.convertIntegerMap(musicGroupPaymentCalenderDao.countActualNum(collect));
		rows.forEach(e -> {
			e.setActualNum(actualNumMap.get(e.getId()) == null ? 0 : actualNumMap.get(e.getId()).intValue());
			e.setExpectNum(expectNumMap.get(e.getId()) == null ? 0 : expectNumMap.get(e.getId()).intValue());
		});
		return pageInfo;
	}

	@Override
	public void paymentPush(Long id, String userIds) {
		// 推送所有
		if (StringUtils.isEmpty(userIds)) {
			List<MusicGroupPaymentCalenderDetail> details = musicGroupPaymentCalenderDetailDao.queryCanPushList(id);
			if (details == null || details.size() == 0) {
				return;
			}
			String musicGroupId = details.get(0).getMusicGroupId();
			Set<Integer> studentIds = details.stream().map(e -> e.getUserId()).collect(Collectors.toSet());
			// 推送消息
			if (studentIds.size() > 0) {
				Map<Integer, String> push = new HashMap<>();
				for (Integer userId : studentIds) {
					push.put(userId, userId + "");
				}
				MusicGroup musicGroup = musicGroupDao.get(musicGroupId);
				String baseUrl = sysConfigDao.findConfigValue(SysConfigService.BASE_API_URL);
				String memo = "4?" + baseUrl + "/#/renew?musicGroupId=" + musicGroupId;
				// 发送续费通知
				sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.STUDENT_PUSH_WAIT_RENEW_MESSAGE, push,
						null, 0, memo, "STUDENT", musicGroup.getName());
			}
		} else {
			// 获取课推送列表
			// 批量推送
			MusicGroupPaymentCalender calender = musicGroupPaymentCalenderDao.get(id);
			Set<String> studentIds = new HashSet<>(Arrays.asList(userIds.split(",")));
			if (studentIds.size() > 0) {
				Map<Integer, String> push = new HashMap<>();
				for (String userId : studentIds) {
					push.put(Integer.parseInt(userId), userId + "");
				}
				MusicGroup musicGroup = musicGroupDao.get(calender.getMusicGroupId());
				String baseUrl = sysConfigDao.findConfigValue(SysConfigService.BASE_API_URL);
				String memo = "4?" + baseUrl + "/#/renew?musicGroupId=" + calender.getMusicGroupId();
				// 发送续费通知
				sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.STUDENT_PUSH_WAIT_RENEW_MESSAGE, push,
						null, 0, memo, "STUDENT", musicGroup.getName());
			}
		}
	}
}
