package com.ym.mec.biz.service.impl;

import com.ym.mec.biz.dal.dao.StudentPaymentOrderDao;
import com.ym.mec.biz.dal.dao.StudentPaymentRouteOrderDao;
import com.ym.mec.biz.dal.dao.SysUserCashAccountDao;
import com.ym.mec.biz.dal.dto.PageInfoOrder;
import com.ym.mec.biz.dal.dto.StudentPaymentRouteOrderDto;
import com.ym.mec.biz.dal.entity.StudentPaymentRouteOrder;
import com.ym.mec.biz.dal.enums.DealStatusEnum;
import com.ym.mec.biz.dal.enums.GroupType;
import com.ym.mec.biz.dal.enums.SporadicChargeTypeEnum;
import com.ym.mec.biz.dal.page.StudentPaymentOrderQueryInfo;
import com.ym.mec.biz.service.StudentPaymentRouteOrderService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.service.IdGeneratorService;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.util.collection.MapUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;

@Service
public class StudentPaymentRouteOrderServiceImpl extends BaseServiceImpl<Long, StudentPaymentRouteOrder> implements StudentPaymentRouteOrderService {

    @Autowired
    private StudentPaymentRouteOrderDao studentPaymentRouteOrderDao;
    @Autowired
    private SysUserCashAccountDao sysUserCashAccountDao;
    @Autowired
    private StudentPaymentOrderDao studentPaymentOrderDao;
    @Autowired
    private IdGeneratorService idGeneratorService;

    @Override
    public BaseDAO<Long, StudentPaymentRouteOrder> getDAO() {
        return studentPaymentRouteOrderDao;
    }

    @Override
    public PageInfoOrder<StudentPaymentRouteOrderDto> OrderQueryPage(StudentPaymentOrderQueryInfo queryInfo) {
        PageInfoOrder<StudentPaymentRouteOrderDto> pageInfo = new PageInfoOrder<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<String, Object>();
        MapUtil.populateMap(params, queryInfo);

        List<StudentPaymentRouteOrderDto> dataList = new ArrayList<>();
        int count = this.findCount(params);

        BigDecimal totalUserBalance = sysUserCashAccountDao.getTotalUserBalance(params);
        pageInfo.setTotalUserBalance(totalUserBalance);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = studentPaymentRouteOrderDao.queryPageOrder(params);

            StudentPaymentRouteOrderDto orderMoneyAmount = studentPaymentRouteOrderDao.getOrderMoneyAmount(params);
            BigDecimal totalActualAmount = orderMoneyAmount.getActualAmount() != null ? orderMoneyAmount.getActualAmount() : BigDecimal.ZERO;
            BigDecimal totalBalancePaymentAmount = orderMoneyAmount.getBalancePaymentAmount() != null ? orderMoneyAmount.getBalancePaymentAmount() : BigDecimal.ZERO;

            params.put("chargeType", SporadicChargeTypeEnum.RECHARGE);
            BigDecimal totalChargeAmount = studentPaymentRouteOrderDao.getChargeAmount(params);
            totalChargeAmount = totalChargeAmount != null ? totalChargeAmount : BigDecimal.ZERO;
            BigDecimal totalAdvanceAmount = totalChargeAmount.subtract(totalBalancePaymentAmount);

            pageInfo.setTotalAdvanceAmount(totalAdvanceAmount);
            pageInfo.setTotalActualAmount(totalActualAmount);
            pageInfo.setTotalRevenueAmount(totalActualAmount.subtract(totalAdvanceAmount));
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public StudentPaymentRouteOrderDto addOutOrder(StudentPaymentRouteOrderDto studentPaymentRouteOrderDto) {
        Date nowDate = new Date();
        String orderNo = idGeneratorService.generatorId("payment") + "";

        studentPaymentRouteOrderDto.setOrderNo(orderNo);
        studentPaymentRouteOrderDto.setGroupType(GroupType.OUTORDER);
        studentPaymentRouteOrderDto.setExpectAmount(studentPaymentRouteOrderDto.getActualAmount());
        studentPaymentRouteOrderDto.setStatus(DealStatusEnum.SUCCESS);
        studentPaymentRouteOrderDto.setCreateTime(nowDate);
        studentPaymentRouteOrderDto.setUpdateTime(nowDate);
        studentPaymentOrderDao.insert(studentPaymentRouteOrderDto);
        StudentPaymentRouteOrder studentPaymentRouteOrder = new StudentPaymentRouteOrder();
        studentPaymentRouteOrder.setOrderNo(orderNo);
        studentPaymentRouteOrder.setRouteOrganId(studentPaymentRouteOrderDto.getOrganId());
        studentPaymentRouteOrder.setRouteAmount(studentPaymentRouteOrderDto.getActualAmount());
        studentPaymentRouteOrder.setRouteBalanceAmount(BigDecimal.ZERO);
        studentPaymentRouteOrder.setFeeFlag("N");
        studentPaymentRouteOrderDao.insert(studentPaymentRouteOrder);
        return studentPaymentRouteOrderDto;
    }

    @Override
    public StudentPaymentRouteOrder addRouteOrder(String orderNo, Integer routeOrganId, BigDecimal routeBalanceAmount) {
        StudentPaymentRouteOrder studentPaymentRouteOrder = new StudentPaymentRouteOrder();
        studentPaymentRouteOrder.setOrderNo(orderNo);
        studentPaymentRouteOrder.setRouteOrganId(routeOrganId);
        studentPaymentRouteOrder.setRouteAmount(BigDecimal.ZERO);
        studentPaymentRouteOrder.setRouteBalanceAmount(routeBalanceAmount);
        studentPaymentRouteOrder.setFeeFlag("N");
        if (studentPaymentRouteOrderDao.insert(studentPaymentRouteOrder) <= 0) {
            throw new BizException("插入订单失败");
        }
        return studentPaymentRouteOrder;
    }

}
