package com.ym.mec.biz.service.impl;

import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.StudentDao;
import com.ym.mec.biz.dal.dao.StudentRepairDao;
import com.ym.mec.biz.dal.dao.SysConfigDao;
import com.ym.mec.biz.dal.dto.BasicUserDto;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.*;
import com.ym.mec.biz.dal.page.RepairStudentQueryInfo;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.service.IdGeneratorService;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.thirdparty.message.MessageSenderPluginContext;
import com.ym.mec.util.collection.MapUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class StudentRepairServiceImpl extends BaseServiceImpl<Integer, StudentRepair> implements StudentRepairService {
    private static final Logger log = LoggerFactory.getLogger(StudentRepairServiceImpl.class);
    @Autowired
    private StudentRepairDao studentRepairDao;
    @Autowired
    private SysUserFeignService sysUserFeignService;
    @Autowired
    private PayService payService;
    @Autowired
    private IdGeneratorService idGeneratorService;
    @Autowired
    private SysConfigDao sysConfigDao;
    @Autowired
    private StudentPaymentOrderService studentPaymentOrderService;
    @Autowired
    private SysUserCashAccountService sysUserCashAccountService;
    @Autowired
    private SysUserCashAccountDetailService sysUserCashAccountDetailService;
    @Autowired
    private SysMessageService sysMessageService;
    @Autowired
    private StudentDao studentDao;
    @Autowired
    private StudentPaymentRouteOrderService studentPaymentRouteOrderService;

    @Override
    public BaseDAO<Integer, StudentRepair> getDAO() {
        return studentRepairDao;
    }

    @Override
    public PageInfo<BasicUserDto> getStudents(RepairStudentQueryInfo queryInfo) {
        PageInfo<BasicUserDto> pageInfo = new PageInfo<>(queryInfo.getPage(), queryInfo.getRows());
        Map<String, Object> params = new HashMap<>();
        MapUtil.populateMap(params, queryInfo);
        List<BasicUserDto> dataList = new ArrayList<>();
        int count = studentRepairDao.getStudentsCount(params);
        if (count > 0) {
            pageInfo.setTotal(count);
            params.put("offset", pageInfo.getOffset());
            dataList = studentRepairDao.getStudents(params);
            Set<Integer> userIds = dataList.stream().map(BasicUserDto::getUserId).collect(Collectors.toSet());
            List<MusicGroup> musicGroups = studentRepairDao.getUserMusicGroup(userIds);
            Map<Integer, List<MusicGroup>> userMusicGroup = musicGroups.stream().collect(Collectors.groupingBy(MusicGroup::getRepairUserId));
            for (BasicUserDto basicUserDto : dataList) {
                if (!userMusicGroup.containsKey(basicUserDto.getUserId())) {
                    continue;
                }
                MusicGroup musicGroup = userMusicGroup.get(basicUserDto.getUserId()).get(0);
                basicUserDto.setMusicGroupId(musicGroup.getId());
                basicUserDto.setMusicGroupName(musicGroup.getName());
            }
        }
        pageInfo.setRows(dataList);
        return pageInfo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Map addRepair(StudentRepair repairInfo) throws Exception {
        studentDao.lockUser(repairInfo.getEmployeeId());
        if (repairInfo.getSendType() != null && repairInfo.getSendType().equals(1) &&
                (repairInfo.getContactName() == null || repairInfo.getContactName().isEmpty()) &&
                (repairInfo.getContactMobile() == null || repairInfo.getContactMobile().isEmpty()) &&
                (repairInfo.getAddress() == null || repairInfo.getAddress().isEmpty())
        ) {
            throw new BizException("邮寄信息必填");
        }
        SysUser student = sysUserFeignService.queryUserById(repairInfo.getStudentId());
        repairInfo.setOrganId(student.getOrganId());

        Date date = new Date();
        BigDecimal amount = repairInfo.getAmount();
        String orderNo = idGeneratorService.generatorId("payment") + "";
        repairInfo.setTransNo(orderNo);
        repairInfo.setRepairStatus(0);
        repairInfo.setPayStatus(0);
        repairInfo.setCreateTime(date);
        repairInfo.setUpdateTime(date);
        studentRepairDao.insert(repairInfo);
        if (repairInfo.getType() == 1) {
            Map<String, Object> repairInfoMap = new HashMap<>();
            MapUtil.populateMap(repairInfoMap, repairInfo);
            return repairInfoMap;
        }
        repairInfo.setPayStatus(1);
        String channelType = "";

        StudentPaymentOrder studentPaymentOrder = new StudentPaymentOrder();
        studentPaymentOrder.setUserId(repairInfo.getStudentId());
        studentPaymentOrder.setGroupType(GroupType.REPAIR);
        studentPaymentOrder.setOrderNo(orderNo);
        studentPaymentOrder.setType(OrderTypeEnum.REPAIR);
        studentPaymentOrder.setExpectAmount(amount);
        studentPaymentOrder.setActualAmount(amount);
        studentPaymentOrder.setStatus(DealStatusEnum.ING);
        studentPaymentOrder.setMusicGroupId(repairInfo.getId().toString());
        studentPaymentOrder.setPaymentChannel("BALANCE");
        studentPaymentOrder.setUpdateTime(date);
        studentPaymentOrder.setOrganId(repairInfo.getOrganId());
        studentPaymentOrder.setRoutingOrganId(repairInfo.getOrganId());

        studentPaymentOrderService.insert(studentPaymentOrder);
        studentPaymentOrder.setVersion(0);

        BigDecimal balance = BigDecimal.ZERO;
        if (repairInfo.getIsUseBalancePayment() && amount.compareTo(BigDecimal.ZERO) > 0) {
            SysUserCashAccount userCashAccount = sysUserCashAccountService.getLocked(repairInfo.getStudentId());
            if (userCashAccount == null) {
                throw new BizException("用户账户找不到");
            }
            if (userCashAccount.getBalance() != null && userCashAccount.getBalance().compareTo(BigDecimal.ZERO) > 0) {
                balance = amount.compareTo(userCashAccount.getBalance()) >= 0 ? userCashAccount.getBalance() : amount;
                amount = amount.subtract(balance);
                studentPaymentOrder.setActualAmount(amount);
                studentPaymentOrder.setBalancePaymentAmount(balance);
                sysUserCashAccountService.updateBalance(repairInfo.getStudentId(), balance.negate(), PlatformCashAccountDetailTypeEnum.PAY_FEE, "乐器维修");
            }
        }
        studentPaymentOrderService.update(studentPaymentOrder);
        studentPaymentOrder.setVersion(studentPaymentOrder.getVersion() + 1);

        if (amount.compareTo(BigDecimal.ZERO) == 0) {
            studentPaymentRouteOrderService.addRouteOrder(orderNo, repairInfo.getOrganId(), balance);
            Map<String, String> notifyMap = new HashMap<>();
            notifyMap.put("tradeState", "1");
            notifyMap.put("merOrderNo", studentPaymentOrder.getOrderNo());
            notifyMap.put("channelType", channelType);
            notifyMap.put("orderNo", "");
            studentPaymentOrderService.updateOrder(notifyMap);
            notifyMap.put("orderNo", orderNo);
            return notifyMap;
        }

        String baseApiUrl = sysConfigDao.findConfigValue(SysConfigService.BASE_API_URL);

        Map<String, BigDecimal> classFee = new HashMap<>();
        classFee.put("course", BigDecimal.ZERO);
        classFee.put("instrument", BigDecimal.ZERO);
        classFee.put("accessories", BigDecimal.ZERO);
        classFee.put("other", amount);

        Map payMap = payService.getPayMap(
                amount,
                balance,
                orderNo,
                baseApiUrl + "/api-student/studentOrder/notify",
                baseApiUrl + "/api-student/studentOrder/paymentResult?type=edu&orderNo=" + orderNo,
                "乐器维修",
                "乐器维修",
                repairInfo.getOrganId(),
                "repair"
        );

        studentPaymentOrder.setMerNos((String) payMap.get("routingMerNos"));
        studentPaymentOrder.setPaymentChannel((String) payMap.get("type"));
        studentPaymentOrder.setUpdateTime(date);
        studentPaymentOrderService.update(studentPaymentOrder);
        return payMap;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void repairSuccess(Integer id, String description, Integer repairStatus) {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            throw new BizException("用户信息获取失败");
        }
        studentDao.lockUser(sysUser.getId());
        StudentRepair studentRepair = studentRepairDao.getRepairInfo(id);
        if (studentRepair == null) {
            throw new BizException("维修信息不存在");
        }
        if (repairStatus != null) {
            studentRepair.setRepairStatus(1);
            studentRepair.setFinishTime(new Date());
        }
        studentRepair.setUpdateTime(new Date());
        studentRepair.setDescription(description);
        studentRepairDao.update(studentRepair);

        if (repairStatus != null) {
            SysUser student = sysUserFeignService.queryUserById(studentRepair.getStudentId());
            Map<Integer, String> map = new HashMap();
            map.put(student.getId(), student.getId().toString());

            String imContent = student.getUsername() + "您的乐器已保养维修完毕，请安排时间来维修点取回\n" +
                    "联系人：" + studentRepair.getEmployeeName() + "（" + studentRepair.getEmployeePhone() + "）\n" +
                    "地址：" + studentRepair.getEmployeeAddress();

            if (studentRepair.getSendType().equals(0)) {
                sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.SMS_REPAIR_UNSEND_COMPLETED, map, null, 0, "1", "STUDENT",
                        student.getUsername(), studentRepair.getEmployeeName(), studentRepair.getEmployeePhone(), studentRepair.getEmployeeAddress());
            } else {
                sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.SMS_REPAIR_SEND_COMPLETED, map, null, 0, "1", "STUDENT",
                        student.getUsername());
                imContent = student.getUsername() + "学员您好,您的乐器已保养维修完毕，快递已寄出，请注意查收。";
            }

            sysMessageService.sendPrivateMessage(student.getId().toString(), imContent);
        }
        log.info("操作人 :" + sysUser.getId());
    }

    @Override
    public BasicUserDto getStudentInfo(Integer studentId) {
        return studentRepairDao.getStudentInfo(studentId);
    }

    @Override
    public StudentRepair getRepairInfo(Integer id) {
        return studentRepairDao.getRepairInfo(id);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Map payRepair(StudentRepair repairInfo) throws Exception {
        if (repairInfo.getSendType().equals(1) &&
                (repairInfo.getContactName() == null || repairInfo.getContactName().isEmpty()) &&
                (repairInfo.getContactMobile() == null || repairInfo.getContactMobile().isEmpty()) &&
                (repairInfo.getAddress() == null || repairInfo.getAddress().isEmpty())
        ) {
            throw new BizException("邮寄信息必填");
        }
        Date date = new Date();
        StudentRepair studentRepair = studentRepairDao.get(repairInfo.getId());
        studentDao.lockUser(studentRepair.getStudentId());
        BigDecimal amount = studentRepair.getAmount();
        String orderNo = idGeneratorService.generatorId("payment") + "";
        studentRepair.setIsUseBalancePayment(repairInfo.getIsUseBalancePayment());
        studentRepair.setTransNo(orderNo);
        studentRepair.setRepairStatus(0);
        studentRepair.setPayStatus(1);
        studentRepair.setSendType(repairInfo.getSendType());
        studentRepair.setContactName(repairInfo.getContactName());
        studentRepair.setContactMobile(repairInfo.getContactMobile());
        studentRepair.setAddress(repairInfo.getAddress());
        studentRepair.setUpdateTime(date);
        studentRepairDao.update(studentRepair);

        String channelType = "";

        StudentPaymentOrder studentPaymentOrder = new StudentPaymentOrder();
        studentPaymentOrder.setUserId(studentRepair.getStudentId());
        studentPaymentOrder.setGroupType(GroupType.REPAIR);
        studentPaymentOrder.setOrderNo(orderNo);
        studentPaymentOrder.setType(OrderTypeEnum.REPAIR);
        studentPaymentOrder.setExpectAmount(amount);
        studentPaymentOrder.setActualAmount(amount);
        studentPaymentOrder.setStatus(DealStatusEnum.ING);
        studentPaymentOrder.setMusicGroupId(studentRepair.getId().toString());
        studentPaymentOrder.setPaymentChannel("BALANCE");
        studentPaymentOrder.setUpdateTime(date);
        studentPaymentOrder.setOrganId(studentRepair.getOrganId());
        studentPaymentOrder.setRoutingOrganId(studentRepair.getOrganId());

        studentPaymentOrderService.insert(studentPaymentOrder);
        studentPaymentOrder.setVersion(0);

        BigDecimal balance = BigDecimal.ZERO;
        if (studentRepair.getIsUseBalancePayment() && amount.compareTo(BigDecimal.ZERO) > 0) {
            SysUserCashAccount userCashAccount = sysUserCashAccountService.getLocked(studentRepair.getStudentId());
            if (userCashAccount == null) {
                throw new BizException("用户账户找不到");
            }
            if (userCashAccount.getBalance() != null && userCashAccount.getBalance().compareTo(BigDecimal.ZERO) > 0) {
                balance = amount.compareTo(userCashAccount.getBalance()) >= 0 ? userCashAccount.getBalance() : amount;
                amount = amount.subtract(balance);
                studentPaymentOrder.setActualAmount(amount);
                studentPaymentOrder.setBalancePaymentAmount(balance);
                sysUserCashAccountService.updateBalance(studentRepair.getStudentId(), balance.negate(), PlatformCashAccountDetailTypeEnum.PAY_FEE, "乐器维修");
            }
        }
        studentPaymentOrderService.update(studentPaymentOrder);
        studentPaymentOrder.setVersion(studentPaymentOrder.getVersion() + 1);

        if (amount.compareTo(BigDecimal.ZERO) == 0) {
            studentPaymentRouteOrderService.addRouteOrder(orderNo, studentRepair.getOrganId(), studentPaymentOrder.getExpectAmount());
            Map<String, String> notifyMap = new HashMap<>();
            notifyMap.put("tradeState", "1");
            notifyMap.put("merOrderNo", studentPaymentOrder.getOrderNo());
            notifyMap.put("channelType", channelType);
            notifyMap.put("orderNo", "");
            studentPaymentOrderService.updateOrder(notifyMap);
            notifyMap.put("orderNo", orderNo);
            return notifyMap;
        }

        String baseApiUrl = sysConfigDao.findConfigValue(SysConfigService.BASE_API_URL);

        Map<String, BigDecimal> classFee = new HashMap<>();
        classFee.put("course", BigDecimal.ZERO);
        classFee.put("instrument", BigDecimal.ZERO);
        classFee.put("accessories", BigDecimal.ZERO);
        classFee.put("other", amount);

        Map payMap = payService.getPayMap(
                amount,
                balance,
                orderNo,
                baseApiUrl + "/api-student/studentOrder/notify",
                baseApiUrl + "/api-student/studentOrder/paymentResult?orderNo=" + orderNo,
                "乐器维修",
                "乐器维修",
                studentRepair.getOrganId(),
                "repair"
        );

        studentPaymentOrder.setMerNos((String) payMap.get("routingMerNos"));
        studentPaymentOrder.setPaymentChannel((String) payMap.get("type"));
        studentPaymentOrder.setUpdateTime(date);
        studentPaymentOrderService.update(studentPaymentOrder);
        return payMap;
    }

    @Override
    public BasicUserDto getStudentRepairer(Integer studentId, Integer organId) {
        BasicUserDto repairer = studentRepairDao.getStudentMusicGroupRepairer(studentId);
        if (repairer == null) {
            repairer = studentRepairDao.getStudentOrganRepairer(organId);
        }
        return repairer;
    }

    @Override
    @Transactional(propagation = Propagation.REQUIRED, rollbackFor = Exception.class)
    public Boolean orderCallback(StudentPaymentOrder studentPaymentOrder) {
        Date nowDate = new Date();
        //更新订单信息
        studentPaymentOrder.setUpdateTime(nowDate);
        int updateCount = studentPaymentOrderService.update(studentPaymentOrder);
        if (updateCount <= 0) {
            throw new BizException("订单更新失败");
        }
        //更新维修单信息
        StudentRepair repairInfo = getRepairInfo(Integer.parseInt(studentPaymentOrder.getMusicGroupId()));
        if (repairInfo == null) {
            throw new BizException("维修单不存在");
        }
        if(repairInfo.getPayStatus().equals(2)){
            return true;
        }

        Integer userId = studentPaymentOrder.getUserId();

        Map<Integer, String> map = new HashMap();
        map.put(userId, userId.toString());

        if (studentPaymentOrder.getStatus() == DealStatusEnum.SUCCESS) {
            repairInfo.setPayStatus(2);
            repairInfo.setUpdateTime(nowDate);
            if (this.update(repairInfo) <= 0) {
                throw new BizException("维修单更新失败");
            }

            //插入交易明细
            BigDecimal amount = studentPaymentOrder.getActualAmount();
            SysUserCashAccount cashAccount = sysUserCashAccountService.get(userId);
            //充值
            SysUserCashAccountDetail rechargeDetail = new SysUserCashAccountDetail();
            rechargeDetail.setAmount(amount);
            rechargeDetail.setBalance(cashAccount.getBalance().add(amount));
            rechargeDetail.setComment("缴费前充值");
            rechargeDetail.setCreateTime(nowDate);
            rechargeDetail.setStatus(DealStatusEnum.SUCCESS);
            rechargeDetail.setTransNo(studentPaymentOrder.getTransNo());
            rechargeDetail.setType(PlatformCashAccountDetailTypeEnum.RECHARGE);
            rechargeDetail.setUpdateTime(nowDate);
            rechargeDetail.setUserId(userId);
            rechargeDetail.setChannel(studentPaymentOrder.getPaymentChannel());
            rechargeDetail.setComAmount(studentPaymentOrder.getComAmount());
            rechargeDetail.setPerAmount(studentPaymentOrder.getPerAmount());
            sysUserCashAccountDetailService.insert(rechargeDetail);

            //缴费
            SysUserCashAccountDetail paymentDetail = new SysUserCashAccountDetail();
            paymentDetail.setAmount(amount.negate());
            paymentDetail.setBalance(cashAccount.getBalance());
            paymentDetail.setComment("乐器维修");
            paymentDetail.setCreateTime(nowDate);
            paymentDetail.setStatus(DealStatusEnum.SUCCESS);
            paymentDetail.setTransNo(studentPaymentOrder.getTransNo());
            paymentDetail.setType(PlatformCashAccountDetailTypeEnum.PAY_FEE);
            paymentDetail.setUpdateTime(nowDate);
            paymentDetail.setUserId(userId);
            rechargeDetail.setChannel(studentPaymentOrder.getPaymentChannel());
            if (studentPaymentOrder.getComAmount() != null) {
                rechargeDetail.setComAmount(studentPaymentOrder.getComAmount().negate());
                rechargeDetail.setPerAmount(studentPaymentOrder.getPerAmount().negate());
            }
            sysUserCashAccountDetailService.insert(paymentDetail);

            String imContent = repairInfo.getStudentName() + "学员您好，您的乐器维修已受理，我们会尽快完成保养维修";

            if (repairInfo.getType().equals(1)) { //线上
                imContent = repairInfo.getStudentName() + "学员您好，请尽快寄送乐器至维修点，我们会尽快完成保养维修\n" +
                        "联系人：" + repairInfo.getEmployeeName() + "（" + repairInfo.getEmployeePhone() + "）\n" +
                        "地址：" + repairInfo.getEmployeeAddress() + "";

                sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.SMS_REPAIR_ONLINE_PAYMENT_SUCCESS, map, null, 0, "1", "STUDENT",
                        repairInfo.getStudentName(), repairInfo.getEmployeeName(), repairInfo.getEmployeePhone(), repairInfo.getEmployeeAddress());
            } else {
                sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.SMS_REPAIR_OFFLINE_PAYMENT_SUCCESS, map, null, 0, "1", "STUDENT",
                        repairInfo.getStudentName());
            }
            sysMessageService.sendNoAuthPrivateMessage(repairInfo.getEmployeeId().toString(), userId.toString(), imContent);
            return true;
        }

        if (studentPaymentOrder.getStatus() == DealStatusEnum.CLOSE || studentPaymentOrder.getStatus() == DealStatusEnum.FAILED) {
            repairInfo.setPayStatus(0);
            repairInfo.setUpdateTime(nowDate);
            if (this.update(repairInfo) <= 0) {
                throw new BizException("维修单更新失败");
            }
            if (studentPaymentOrder.getBalancePaymentAmount() != null && studentPaymentOrder.getBalancePaymentAmount().compareTo(BigDecimal.ZERO) > 0) {
                sysUserCashAccountService.updateBalance(studentPaymentOrder.getUserId(), studentPaymentOrder.getBalancePaymentAmount(), PlatformCashAccountDetailTypeEnum.REFUNDS, "乐器维修支付失败");
            }

            sysMessageService.batchSendMessage(MessageSenderPluginContext.MessageSender.JIGUANG, MessageTypeEnum.SMS_SPORADIC_PAYMENT_FAILED, map, null, 0, "", "STUDENT",
                    studentPaymentOrder.getActualAmount(), "乐器维修");
            return false;
        }
        return false;
    }

}
