package com.ym.mec.student.controller;


import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.EmployeeDao;
import com.ym.mec.biz.dal.entity.Employee;
import com.ym.mec.biz.dal.entity.StudentRepair;
import com.ym.mec.biz.dal.page.RepairStudentQueryInfo;
import com.ym.mec.biz.service.StudentRepairService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Objects;

@RequestMapping("repair")
@Api(tags = "学生维修服务")
@RestController
public class RepairController extends BaseController {

    @Autowired
    private SysUserFeignService sysUserFeignService;

    @Autowired
    private StudentRepairService studentRepairService;

    @ApiOperation("获取维修记录")
    @GetMapping(value = "/getStudentRepairList")
    public HttpResponseResult getStudentRepairList(RepairStudentQueryInfo queryInfo) {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed("用户信息获取失败");
        }
        queryInfo.setStudentId(sysUser.getId());
        queryInfo.setPayStatus(2);
        return succeed(studentRepairService.queryPage(queryInfo));
    }

    @ApiOperation("获取维修记录详情")
    @GetMapping(value = "/getRepairInfo")
    public HttpResponseResult getRepairInfo(Integer id,Integer studentId) {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if ((sysUser == null || sysUser.getId() ==null) && studentId == null) {
            return failed("用户信息获取失败");
        }
        if(sysUser.getId() != null){
            studentId = sysUser.getId();
        }
        StudentRepair repairInfo = studentRepairService.getRepairInfo(id);
        if (!repairInfo.getStudentId().equals(studentId)) {
            return failed("您的维修记录不存在");
        }
        return succeed(repairInfo);
    }

    @ApiOperation("支付维修单")
    @PostMapping(value = "/payRepair")
    public HttpResponseResult payRepair(StudentRepair repairInfo) throws Exception {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed(HttpStatus.FORBIDDEN, "请登录");
        }
        return succeed(studentRepairService.payRepair(repairInfo));
    }

    @ApiOperation("获取维修技师信息")
    @PostMapping(value = "/getRepairer")
    public HttpResponseResult getRepairer() {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed(HttpStatus.FORBIDDEN, "请登录");
        }
        return succeed(studentRepairService.getStudentRepairer(sysUser.getId(),sysUser.getOrganId()));
    }


}
