package com.ym.mec.web.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.ym.mec.biz.dal.entity.MusicGroupPaymentCalender;
import com.ym.mec.biz.dal.page.MusicCalenderQueryInfo;
import com.ym.mec.biz.service.MusicGroupPaymentCalenderService;
import com.ym.mec.common.controller.BaseController;

@RequestMapping("musicGroupPaymentCalender")
@Api(tags = "乐团缴费日历服务")
@RestController
public class MusicGroupPaymentCalenderController extends BaseController {

    @Autowired
    private MusicGroupPaymentCalenderService musicGroupPaymentCalenderService;

    @ApiOperation(value = "新增乐团缴费日历")
    @PostMapping("/add")
    @PreAuthorize("@pcs.hasPermissions('musicGroupPaymentCalender/add')")
    public Object add(MusicGroupPaymentCalender musicGroupPaymentCalender) {
        musicGroupPaymentCalenderService.create(musicGroupPaymentCalender);
        return succeed();
    }

    @ApiOperation(value = "批量新增、修改乐团缴费周期")
    @PostMapping("/batchAdd")
    @PreAuthorize("@pcs.hasPermissions('musicGroupPaymentCalender/batchAdd')")
    public Object batchAdd(@RequestBody List<MusicGroupPaymentCalender> musicGroupPaymentCalenders) {
        musicGroupPaymentCalenderService.batchInsert(musicGroupPaymentCalenders);
        return succeed();
    }

    @ApiOperation(value = "删除乐团缴费日历")
    @PostMapping("/del")
    @PreAuthorize("@pcs.hasPermissions('musicGroupPaymentCalender/del')")
    public Object del(Long id) {
        musicGroupPaymentCalenderService.del(id);
        return succeed();
    }

    @ApiOperation(value = "修改乐团缴费日历")
    @PostMapping("/updateStartTime")
    @PreAuthorize("@pcs.hasPermissions('musicGroupPaymentCalender/updateStartTime')")
    public Object updateStartTime(Date startTime,Date endTime,Long id) {
        musicGroupPaymentCalenderService.updateStartTime(startTime,endTime,id);
        return succeed();
    }

    @ApiOperation(value = "分页查询乐团缴费日历列表")
    @GetMapping("/queryPage")
    @PreAuthorize("@pcs.hasPermissions('musicGroupPaymentCalender/queryPage')")
    public Object queryPage(MusicCalenderQueryInfo queryInfo) {
        return succeed(musicGroupPaymentCalenderService.queryDetailPage(queryInfo));
    }

    @ApiOperation(value = "获取缴费信息")
    @GetMapping("/getDetail")
    @PreAuthorize("@pcs.hasPermissions('musicGroupPaymentCalender/getDetail')")
    public Object getDetail(Long id) {
        return succeed(musicGroupPaymentCalenderService.getDetail(id));
    }

    @ApiOperation(value = "开启缴费的推送(暂时不用，没写完)")
    @GetMapping("/paymentPush")
    @PreAuthorize("@pcs.hasPermissions('musicGroupPaymentCalender/paymentPush')")
    public Object paymentPush(Long id,String userIds) {
        musicGroupPaymentCalenderService.paymentPush(id,userIds);
        return succeed();
    }
}
