package com.ym.mec.web.controller.education;


import com.ym.mec.auth.api.client.SysUserFeignService;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.EmployeeDao;
import com.ym.mec.biz.dal.dto.BasicUserDto;
import com.ym.mec.biz.dal.entity.Employee;
import com.ym.mec.biz.dal.entity.StudentRepair;
import com.ym.mec.biz.dal.page.RepairStudentQueryInfo;
import com.ym.mec.biz.service.StudentRepairService;
import com.ym.mec.biz.service.SubjectService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.util.date.DateUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.snaker.engine.access.Page;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Objects;

@RequestMapping("eduRepair")
@Api(tags = "教务维修服务")
@RestController
public class EduRepairController extends BaseController {

    @Autowired
    private SysUserFeignService sysUserFeignService;

    @Autowired
    private EmployeeDao employeeDao;
    @Autowired
    private StudentRepairService studentRepairService;
    @Autowired
    private SubjectService subjectService;

    @ApiOperation("获取学生列表")
    @GetMapping(value = "/getStudents")
    public HttpResponseResult<PageInfo<BasicUserDto>> getStudents(RepairStudentQueryInfo queryInfo) {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed(HttpStatus.FORBIDDEN, "请登录");
        }

        Employee employee = employeeDao.get(sysUser.getId());
        if (Objects.isNull(employee)) {
            return failed(HttpStatus.NOT_ACCEPTABLE, "员工信息不存在");
        }
        queryInfo.setEmployeeId(sysUser.getId());
        queryInfo.setOrganIdList(employee.getOrganIdList());
        return succeed(studentRepairService.getStudents(queryInfo));
    }

    @ApiOperation("添加维修单")
    @PostMapping(value = "/addRepair")
    public HttpResponseResult addRepair(StudentRepair repairInfo) throws Exception {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed(HttpStatus.FORBIDDEN, "请登录");
        }
        repairInfo.setEmployeeId(sysUser.getId());
        repairInfo.setEmployeeName(sysUser.getRealName());
        return succeed(studentRepairService.addRepair(repairInfo));
    }

    @ApiOperation("获取学生的维修记录")
    @GetMapping(value = "/getStudentRepairList")
    public HttpResponseResult getStudentRepairList(RepairStudentQueryInfo queryInfo) {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed("用户信息获取失败");
        }
        queryInfo.setEmployeeId(sysUser.getId());
        queryInfo.setPayStatus(2);
        return succeed(studentRepairService.queryPage(queryInfo));
    }

    @ApiOperation("获取乐器种类")
    @GetMapping(value = "/findSubSubjects")
    public HttpResponseResult findSubSubjects() {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed("用户信息获取失败");
        }
        return succeed(subjectService.findSubSubjects(1));
    }

    @ApiOperation("获取学生信息")
    @GetMapping(value = "/getStudentInfo")
    public HttpResponseResult getStudentInfo(Integer studentId) {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed("用户信息获取失败");
        }
        return succeed(studentRepairService.getStudentInfo(studentId));
    }

    @ApiOperation("获取维修记录详情")
    @GetMapping(value = "/getRepairInfo")
    public HttpResponseResult getRepairInfo(Integer id) {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed("用户信息获取失败");
        }
        return succeed(studentRepairService.getRepairInfo(id));
    }


    @ApiOperation("获取维修列表")
    @GetMapping(value = "/queryPage")
    public HttpResponseResult queryPage(RepairStudentQueryInfo queryInfo) {
        SysUser sysUser = sysUserFeignService.queryUserInfo();
        if (sysUser == null) {
            return failed("用户信息获取失败");
        }
        queryInfo.setEmployeeId(sysUser.getId());
        Date endTime = queryInfo.getEndTime();
        if(endTime != null){
            queryInfo.setEndTime(DateUtil.addDays(endTime, 1));
        }
        queryInfo.setPayStatus(2);
        return succeed(studentRepairService.queryPage(queryInfo));
    }


    @ApiOperation("维修完成")
    @PostMapping(value = "/repairSuccess")
    public HttpResponseResult repairSuccess(Integer id,String description,Integer repairStatus) {
        studentRepairService.repairSuccess(id,description,repairStatus);
        return succeed();
    }

}
